#include "StdAfx.h"
#include "ChatMainView.h"
#include "ChatDlg.h"
#include "ChatItem.h"
#include "StickerGroup.h"
CChatMainView::CChatMainView(void)
{
	
	m_bHaveLastMsg = FALSE;
	m_bShowRecentMsg = FALSE;
	m_pChatDlg = NULL;
	m_pChatModel = NULL;

	m_pImgBG = NULL;
	m_pImgSysMsgBG = NULL;
	m_pImgSysMsgArrow = NULL;
	m_pImgSendFail = NULL;

	m_hLinkCursor = AfxGetApp()->LoadStandardCursor(IDC_HAND);
	m_hIbeamCursor = AfxGetApp()->LoadStandardCursor(IDC_IBEAM);
	m_nCursor = CURSOR_DEFAULT;
	ZeroMemory(m_pImgDateBG,sizeof(m_pImgDateBG));
	m_szTotalViewSize.SetSize(0,0);
}

CChatMainView::CChatMainView(CChatMainView& src)
{
	m_pChatDlg = src.GetChatDlg();
	m_pChatModel = src.GetChatModel();

	m_pImgBG = NULL;
	m_pImgSysMsgBG = NULL;
	m_pImgSysMsgArrow = NULL;
	m_pImgSendFail = NULL;

	m_hLinkCursor = AfxGetApp()->LoadStandardCursor(IDC_HAND);
	m_hIbeamCursor = AfxGetApp()->LoadStandardCursor(IDC_IBEAM);
	m_nCursor = CURSOR_DEFAULT;
	m_bHaveLastMsg = FALSE;
	m_bShowRecentMsg = FALSE;
}

CChatMainView::~CChatMainView(void)
{
	if (m_hLinkCursor)
	{
		DestroyCursor(m_hLinkCursor);
		m_hLinkCursor = NULL;
	}

	if (m_hIbeamCursor)
	{
		DestroyCursor(m_hIbeamCursor);
		m_hIbeamCursor = NULL;
	}

	SAFE_DELETE(m_pImgBG);
	SAFE_DELETE(m_pImgDateBG[0]);
	SAFE_DELETE(m_pImgDateBG[1]);
	SAFE_DELETE(m_pImgDateBG[2]);
	SAFE_DELETE(m_pImgSysMsgBG);
	SAFE_DELETE(m_pImgSysMsgArrow);
	SAFE_DELETE(m_pImgSendFail);
}

void CChatMainView::EnableLastMessage(BOOL bHaveLastMsg)
{
	m_bHaveLastMsg = bHaveLastMsg;
}



void CChatMainView::SetChatModel(CChatModel* pChatModel)
{
	if (pChatModel)
	{
		m_pChatModel = pChatModel;
	}
}

void CChatMainView::AddChatItem(CHAT_MSG* pChatMsg)
{
	CTime time(pChatMsg->time);
	GetLocaleTime(&time);

	ChatItemInfo stChatItemfo;
	stChatItemfo.nUserCount = 0;	//don't use.
	stChatItemfo.strUserID = pChatMsg->strID;
	stChatItemfo.strUserName = pChatMsg->strName;
	stChatItemfo.strNameTalkName = pChatMsg->strNickName;
	stChatItemfo.nTid            = pChatMsg->nTid;
	stChatItemfo.nMsgSeq         = pChatMsg->nMsgSeq;
	stChatItemfo.cf              = pChatMsg->cf;
	stChatItemfo.strMsg          = pChatMsg->pszMsg;
	stChatItemfo.time            = time;
	stChatItemfo.eMsgGB			 = CHAT_MSG_NONE;

	switch (pChatMsg->msgType)
	{
	case DEF_CHAT_MSG_TYPE_CHAT:
		{
			stChatItemfo.eMsgGB = CHAT_MSG_CHAT;
			break;
		}
	case DEF_CHAT_MSG_TYPE_STICKER:
		{
			stChatItemfo.eMsgGB = CHAT_MSG_STICKER;
			break;
		}
	case DEF_FTP_MSG_TYPE:
		{
			stChatItemfo.eMsgGB = CHAT_MSG_FTP;
			break;
		}
	case DEF_RECOVER_SESSION:
		{
			stChatItemfo.eMsgGB = CHAT_MSG_RECOVER_SESSION;
			break;
		}
	default:
		{
			if (stChatItemfo.strUserID.IsEmpty())
			{
				stChatItemfo.eMsgGB = CHAT_MSG_SYS;
			}
		}
	}

	AddChatItem(&stChatItemfo, true);
}

void CChatMainView::ShowRecentMessage()
{
	
}

CChatItem* CChatMainView::AllocChatItem(LPChatItemInfo pInfo)
{
	if (pInfo != NULL)
	{
		switch (pInfo->eMsgGB)
		{
		case CHAT_MSG_SYS:
		case CHAT_MSG_CHAT:
		case CHAT_MSG_RECOVER_SESSION:
			{
				return new CChatMsgItem(*pInfo);
			}
		case CHAT_MSG_STICKER:
			{
				return new CChatStickerItem(*pInfo);
			}
		case  CHAT_MSG_FTP:
			{
				return new CChatFileItem(*pInfo);
			}
		case CHAT_MSG_JOIN:
		case CHAT_MSG_LEAVE:
			{
				return new CChatUserJoinItem(*pInfo);
			}
		}
	}

	return NULL;
}

//BOOL CChatMainView::IsDuplicateSystemMessage(CChatItem* pChItem, CChatItem* pChItemLast, CString strChatMsg)
//{
//	if (NULL == pChItem || NULL == pChItemLast)
//		return FALSE;
//	
//	return TRUE;
//	//return pChItemLast && pChItem->eMsgGB == CHAT_MSG_SYS && pChItemLast->eMsgGB == CHAT_MSG_SYS && pChItemLast->strMsg.CompareNoCase(strChatMsg) == 0;
//}

BOOL CChatMainView::IsEmptySystemMessage( CChatItem* pChItem, CString strChatMsg )
{
	//return (pChItem->eMsgGB == CHAT_MSG_SYS && strChatMsg.IsEmpty());
	return pChItem->IsSysType() && strChatMsg.IsEmpty();
}

Image* CChatMainView::GetStickerImage(CString strStickerID)
{
	int pos = strStickerID.Find('_');
	if(pos == -1)
	{
		return NULL;
	}

	Image *pImage = NULL;
	int grpid = _wtoi(strStickerID.Left(pos + 1));
	int sid = _wtoi(strStickerID.Right(strStickerID.GetLength() - pos - 1));
	CStickerMgr &stickerMgr = GET_CONFIG()->GetStickerMgr();
	CStickerGroup *pStickerGroup = stickerMgr.GetGroup(grpid);
	if (pStickerGroup)
	{
		for (int i = 0; i < pStickerGroup->GetStickerCount(); i++)
		{
			CStickerItem *psi = pStickerGroup->GetStickerItem(i);
			if (psi->GetStickerId() == sid)
			{
				pImage = psi->GetImagePtr();
				break;
			}
		}
	}
// 	for (DWORD i = 0; i < m_pChatDlg->GetStickerVector()->size(); i++)
// 	{
// 		AllStickerVec vec= *(m_pChatDlg->GetStickerVector());
// 		CStickerItem* pItem = (CStickerItem*)vec[i];
// 
// 		if (!pItem)
// 			continue;
// 
// 		if (pItem->m_strStickerID.CompareNoCase(strStickerID) == 0)
// 			return pItem->m_img;
// 	}

	return pImage;
}

void CChatMainView::SetImgBGPath(CString strFileName)
{
	if (strFileName.IsEmpty())
		return;

	m_pImgBG = new Image(GetImagePath(strFileName));
}

void CChatMainView::SetImgDateBGPath(CString strFileName, int nIndex)
{
	if (strFileName.IsEmpty() || nIndex < 0 || nIndex >= 3)
		return;

	m_pImgDateBG[nIndex] = new Image(GetImagePath(strFileName));
}

void CChatMainView::SetImgSysMsgBGPath(CString strFileName)
{
	if (strFileName.IsEmpty())
		return;

	m_pImgSysMsgBG = new Image(GetImagePath(strFileName));
}

void CChatMainView::SetImgSysMsgArrow(CString strFileName)
{
	if (strFileName.IsEmpty())
		return;

	m_pImgSysMsgArrow = new Image(GetImagePath(strFileName));
}

void CChatMainView::SetImgSendFail(CString strFileName)
{
	if (strFileName.IsEmpty())
		return;

	m_pImgSendFail = new Image(GetImagePath(strFileName));
}

CString CChatMainView::GetImagePath( CString strFileName )
{
	CString strImgPath = _T(""), strImgDir = _T("");
	strImgDir = GetAppImgDir();
	strImgPath.Format(_T("%s\\%s"), strImgDir, strFileName);

	return strImgPath;
}

// ޽  URL ڿ ġ ã´.
//void CChatMainView::ParseUrlInMsg(CChatItem* pChItem)
//{
//	CString strMsg = pChItem->strMsg;
//	strMsg.MakeLower();
//
//	URL_POS urlPos;
//
//	// URL Ǵ Ű. Ϲ ͸ ߰ , ʿ信  ߰ϸ ȴ.
//	const UINT uKeywords = 8;
//	CString strKeyword[uKeywords]= {_T("http://"), 
//		_T("https://"), 
//		_T("ftp://"),
//		_T("www."), 
//		_T(".co.kr"),
//		_T(".com"), 
//		_T(".net"), 
//		_T(".org")};
//
//	int nLength = strMsg.GetLength();
//
//	CString strTokenize;
//	CString strDelimeter(_T("\" \' \r \n \t"));
//	int nCurPos = 0;
//
//	strTokenize = strMsg.Tokenize(strDelimeter, nCurPos);
//	while(strTokenize != _T(""))
//	{
//		for (int i = 0; i < uKeywords; i++)
//		{
//			int nFind = strTokenize.Find(strKeyword[i]);
//
//			if (nFind > -1)
//			{
//				pChItem->bUrl = TRUE;
//				urlPos.nStart = nCurPos - strTokenize.GetLength() - 1;
//				urlPos.nEnd = nCurPos - 1;
//				pChItem->UrlPosVector.push_back(urlPos);
//
//				//URL ȮϹǷ ߰ Ľ ʿ
//				if (i < 3)
//				{
//					break;
//				}
//			}
//		}
//
//		strTokenize = strMsg.Tokenize(strDelimeter, nCurPos);
//	}
//}
